/*
 *     Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 *     Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
import java.net.URL
import org.danilopianini.gradle.gitsemver.UpdateType
import org.jetbrains.dokka.DokkaConfiguration
import org.jetbrains.dokka.base.DokkaBase
import org.jetbrains.dokka.base.DokkaBaseConfiguration
import org.jetbrains.dokka.gradle.DokkaTask
import org.jetbrains.kotlin.gradle.ExperimentalKotlinGradlePluginApi

plugins {
    alias(libs.plugins.kotlinMultiplatform)
    alias(libs.plugins.androidLibrary)
    alias(libs.plugins.ksp)
    alias(libs.plugins.ktorfit)
    alias(libs.plugins.kotlinxSerialization)
    alias(libs.plugins.kotestMultiplatform)
    alias(libs.plugins.spotless)
    alias(libs.plugins.gitSemVer)
    alias(libs.plugins.dataframe)
    id("maven-publish")
    id("signing")
    alias(libs.plugins.dokka)
}

buildscript {
    dependencies {
        classpath(libs.dokka.base)
    }
}

gitSemVer {
    developmentIdentifier.set("dev")
    fullHash.set(false)
    developmentCounterLength.set(2)
    enforceSemanticVersioning.set(true)
    commitNameBasedUpdateStrategy { UpdateType.MINOR }
}

kotlin {
    jvmToolchain(21)
    applyDefaultHierarchyTemplate()
    explicitApi()

    @OptIn(ExperimentalKotlinGradlePluginApi::class)
    compilerOptions {
        freeCompilerArgs.add("-Xexpect-actual-classes")
    }

    jvm()
    androidTarget()

    sourceSets {
        all {
            languageSettings.apply {
                optIn("kotlin.RequiresOptIn")
            }
        }
        val commonMain by getting {
            dependencies {
                api(libs.ktor.client.core)
                implementation(libs.bundles.ktor)
                implementation(libs.bundles.sandwich)
                api(libs.kotlinx.datetime)
                implementation(libs.koin.core)
                implementation(libs.ktorfit.lib)
                implementation(libs.kotlinx.coroutines.core)
                implementation(libs.kermit)
            }
        }
        val commonTest by getting {
            dependencies {
                implementation(libs.kotest.framework.engine)
                implementation(libs.kotest.assertions.core)
                implementation(libs.kotest.framework.datatest)
                implementation(libs.ktor.test)
            }
        }
        val jvmMain by getting {
            dependencies {
                implementation(libs.ktor.client.java)
                implementation(libs.slf4j)
            }
        }
        val androidMain by getting {
            dependencies {
                implementation(libs.ktor.client.android)
            }
        }
        val jvmTest by getting {
            dependencies {
                implementation(libs.dotenv.vault)

                implementation(libs.kotest.runner.junit5)
                implementation(libs.kotlinx.coroutines.slf4j)
                implementation(libs.kotlinx.coroutines.debug)
                implementation(libs.kotlinx.dataframe)
                // Required to parse dataframe rows back to JSON objects
                implementation(libs.klaxon)
            }
        }
    }
}

android {
    compileSdk = libs.versions.compileSdk.get().toInt()
    defaultConfig {
        minSdk = libs.versions.minSdk.get().toInt()
        testInstrumentationRunner = "androidx.test.runner.AndroidJUnitRunner"
    }
    namespace = "edu.ucsc.its.temerity"
}

spotless {
    kotlin {
        target("**/*.kt")
        targetExclude("$buildDir/**/*.kt")
        ktlint().editorConfigOverride(
            mapOf(
                "indent_size" to "2",
                "continuation_indent_size" to "2"
            )
        )
        licenseHeaderFile(rootProject.file("spotless/libHeader.kt"))
        trimTrailingWhitespace()
        endWithNewline()
    }
    format("kts") {
        target("**/*.kts")
        targetExclude("$buildDir/**/*.kts")
        licenseHeaderFile(rootProject.file("spotless/libHeader.kt"), "(^(?![\\/ ]\\*).*$)")
        trimTrailingWhitespace()
        endWithNewline()
    }
}

tasks.withType<DokkaTask>().configureEach {
    pluginConfiguration<DokkaBase, DokkaBaseConfiguration> {
        footerMessage = "Developed by William Walker Copyright © 2024 Regents of the University of California"
    }

    dokkaSourceSets {
        named("commonMain") {
            moduleName.set("Temerity")
            sourceRoots.from(file("src/commonMain"))
            skipEmptyPackages.set(true)
            documentedVisibilities.set(
                setOf(DokkaConfiguration.Visibility.PUBLIC, DokkaConfiguration.Visibility.INTERNAL)
            )

            sourceLink {
                localDirectory.set(file("src/commonMain/kotlin"))
                remoteUrl.set(
                    URL("https://git.ucsc.edu/wnwalker/temerity/-/tree/main/temerity/src/commonMain/kotlin/edu/ucsc/its/temerity")
                )
                remoteLineSuffix.set("#L")
            }
        }
    }
}

signing {
    val signingKeyId: String? by project
    val signingKey: String? by project
    val signingPassword: String? by project
    if (!signingKey.isNullOrEmpty()){
        useInMemoryPgpKeys(signingKeyId, signingKey, signingPassword)
        publishing.publications {
            sign(publishing.publications)
        }
    }
}

publishing {
    repositories {
        maven {
            name = "Temerity"
            url = uri("https://git.ucsc.edu/api/v4/projects/12162/packages/maven")
            credentials(HttpHeaderCredentials::class) {
                name = "Job-Token"
                value = System.getenv("CI_JOB_TOKEN")
            }
            authentication {
                create("header", HttpHeaderAuthentication::class)
            }
        }
    }
    publications {
        withType<MavenPublication> {
            // Workaround for gradle issue: https://github.com/gradle/gradle/issues/26091
            // https://github.com/voize-gmbh/reakt-native-toolkit/commit/baf0392b65e3ab0305b491f64c45a3a5fc6c86a4
            val publication = this
            val dokkaJar = tasks.register<Jar>("${publication.name}DokkaJar") {
                group = JavaBasePlugin.DOCUMENTATION_GROUP
                description = "Assembles Kotlin docs with Dokka into a Javadoc jar"
                archiveClassifier.set("javadoc")
                from(tasks.named("dokkaHtml"))
                // Each archive name should be distinct, to avoid implicit dependency issues.
                // We use the same format as the sources Jar tasks.
                // https://youtrack.jetbrains.com/issue/KT-46466
                archiveBaseName.set("${archiveBaseName.get()}-${publication.name}")
            }

            artifact(dokkaJar)

            // Version set by Gradle Git SemVer plugin: https://github.com/DanySK/git-sensitive-semantic-versioning-gradle-plugin
            group = "edu.ucsc.its"

            pom {
                name.set("Temerity Library")
                description.set("A Kotlin Multiplatform library for interacting with the YuJa EVP API")
                url.set("https://git.ucsc.edu/wnwalker/temerity")

                licenses {
                    license {
                        name.set("GNU Lesser General Public License, Version 2.1")
                        url.set("https://www.gnu.org/licenses/old-licenses/lgpl-2.1.txt")
                    }
                }
                developers {
                    developer {
                        id.set("wnwalker")
                        name.set("William Walker")
                        email.set("wnwalker@ucsc.edu")
                        organizationUrl.set("https://its.ucsc.edu")
                    }
                }

                issueManagement {
                    system.set("GitLab")
                    url.set("https://git.ucsc.edu/wnwalker/temerity/-/issues")
                }

                scm {
                    connection.set("scm:git:https://git.ucsc.edu/wnwalker/temerity.git")
                    developerConnection.set("scm:git:ssh://git@git.ucsc.edu:wnwalker/temerity.git")
                    url.set("https://git.ucsc.edu/wnwalker/temerity")
                }
            }
        }
    }
}
