/*
 *     Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 *     Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package edu.ucsc.its.temerity.remote

import edu.ucsc.its.temerity.LocalDateTimeSerializer
import kotlinx.datetime.LocalDate
import kotlinx.datetime.LocalDateTime
import kotlinx.serialization.SerialName
import kotlinx.serialization.Serializable

/*
 * Classes representing models used by the users API
 * https://support.yuja.com/hc/en-us/articles/360049580714-YuJa-API#h_01H8YCQKHZB4Q5S9GC1P7WWGG8
 */
@Serializable
public data class User(
  @SerialName("login_id")
  val loginId: String = "",

  @SerialName("email_address")
  val emailAddress: String = "",

  @SerialName("user_type")
  val userType: String = "",

  @SerialName("user_id")
  val userId: Long = -1,

  val timezone: String = "",

  @SerialName("custom_id")
  val customId: String = "",

  @SerialName("last_name")
  var lastName: String = "",

  @SerialName("phone_number")
  val phoneNumber: String = "",

  @SerialName("first_name")
  val firstName: String = "",
)

@Serializable
public data class NewUser(
  @SerialName("email_address")
  val emailAddress: String = "",

  @SerialName("first_name")
  val firstName: String = "",

  @SerialName("last_name")
  val lastName: String = "",

  @SerialName("login_id")
  val loginId: String = "",

  @SerialName("login_password")
  val loginPassword: String = "",

  @SerialName("custom_id")
  val customId: String = "",

  @SerialName("user_type")
  val userType: String = "",

  @SerialName("phone_number")
  val phoneNumber: String = "",

  @SerialName("primary_identifier")
  val primaryIdentifier: String = "",

  @SerialName("secondary_identifier")
  val secondaryIdentifier: String = "",
) {
  public fun validate(): String {
    return when {
      emailAddress.isBlank() -> "Email address is required"
      firstName.isBlank() -> "First name is required"
      lastName.isBlank() -> "Last name is required"
      loginId.isBlank() -> "Login ID is required"
      loginPassword.isBlank() -> "Login password is required"
      userType.isBlank() -> "User type is required"
      else -> "Valid"
    }
  }
}

/**
 * @param userType The type of user to create. The different types of users you can create are: "Student", "Instructor", "IT Manager"
 */
@Serializable
public data class UserUpdate(
  @SerialName("email_address")
  var emailAddress: String? = null,

  @SerialName("first_name")
  var firstName: String? = null,

  @SerialName("last_name")
  var lastName: String? = null,

  @SerialName("login_password")
  var loginPassword: String? = null,

  @SerialName("custom_id")
  var customId: String? = null,

  @SerialName("timezone")
  var timezone: String? = null,

  @SerialName("user_type")
  var userType: String? = null,

  @SerialName("phone_number")
  var phoneNumber: String? = null,
)

@Serializable
public data class UserGroup(
  @SerialName("publish_type")
  val publishType: String = "",

  @SerialName("is_active")
  val isActive: Int = -1,

  @SerialName("group_code")
  val groupCode: String = "",

  @SerialName("group_id")
  val groupId: Long = -1,

  @SerialName("group_name")
  val groupName: String = "",

  @SerialName("owner_id")
  val ownerId: Long = -1,

  @SerialName("group_security")
  val groupSecurity: String = "",

  @SerialName("group_term")
  val groupTerm: String = "",
)

/*
 * The different types of folder permission you can set are:
 *  - "Role"
 *  - "User"
 *  - "Group"
 */
@Serializable
public object FolderPermissions {
//  public interface Action
  public sealed class Action {
    public class AddRole(
      public val folderId: Long,
      public val type: Type = FolderPermissions.Type.ROLE,
      public val name: Roles,
      public val updateDescendants: Boolean = true,
      public val access: AccessLevel,
    ) : Action()
    public class AddUser(
      public val folderId: Long,
      public val type: Type = FolderPermissions.Type.USER,
      public val updateDescendants: Boolean = true,
      public val userName: String,
      public val access: AccessLevel,
    ) : Action()
    public class AddGroup(
      public val folderId: Long,
      public val type: Type = Type.GROUP,
      public val updateDescendants: Boolean = true,
      public val groupName: String,
      public val access: AccessLevel,
    ) : Action()
    public class RemoveRole(
      public val folderId: Long,
      public val type: Type = Type.ROLE,
      public val name: Roles,
    ) : Action()
    public class RemoveUser(
      public val folderId: Long,
      public val type: Type = Type.USER,
      public val userName: String,
    ) : Action()
    public class RemoveGroup(
      public val folderId: Long,
      public val type: Type = Type.GROUP,
      public val groupName: String,
    ) : Action()
  }

  @Serializable
  public enum class Type(@Suppress("unused") public val value: String) {
    ROLE("Role"),
    USER("User"),
    GROUP("Group"),
  }

  @Serializable
  public enum class Roles(@Suppress("unused") public val value: String) {
    STUDENT("Student"),
    INSTRUCTOR("Instructor"),
  }

  @Serializable
  public enum class AccessLevel(@Suppress("unused") public val value: String) {
    FULL_ACCESS("Full Access"),
    EDIT_ACCESS("Edit Access"),
    EDIT_CAPTION_ACCESS("Edit Caption Access"),
    READ_ACCESS("Read Access"),
  }
}

// @Serializable
// public data class AddRoleFolderPermissions(
//  val folderId: Long,
//  val type: FolderPermissions.Type = FolderPermissions.Type.ROLE,
//  val name: FolderPermissions.Roles,
//  val updateDescendants: Boolean = true,
//  val access: FolderPermissions.AccessLevel,
// ) : FolderPermissions.Action

// @Serializable
// public data class AddUserFolderPermissions(
//  val folderId: Long,
//  val type: FolderPermissions.Type = FolderPermissions.Type.USER,
//  val updateDescendants: Boolean = true,
//  val userName: String,
//  val access: FolderPermissions.AccessLevel,
// ) : FolderPermissions.Action

// @Serializable
// public data class AddGroupFolderPermissions(
//  val folderId: Long,
//  val type: FolderPermissions.Type = FolderPermissions.Type.GROUP,
//  val updateDescendants: Boolean = true,
//  val groupName: String,
//  val access: FolderPermissions.AccessLevel,
// ) : FolderPermissions.Action

// @Serializable
// public data class RemoveRoleFolderPermissions(
//  val folderId: Long,
//  val type: FolderPermissions.Type = FolderPermissions.Type.ROLE,
//  val name: FolderPermissions.Roles,
// ) : FolderPermissions.Action

// @Serializable
// public data class RemoveUserFolderPermissions(
//  val folderId: Long,
//  val type: FolderPermissions.Type = FolderPermissions.Type.USER,
//  val userName: String,
// ) : FolderPermissions.Action

// @Serializable
// public data class RemoveGroupFolderPermissions(
//  val folderId: Long,
//  val type: FolderPermissions.Type = FolderPermissions.Type.GROUP,
//  val groupName: String,
// ) : FolderPermissions.Action

@Serializable
public data class LinkageValue(
  val linkage: String,
)

/*
 * Classes representing models used by the groups API
 * https://support.yuja.com/hc/en-us/articles/360049580714-YuJa-API#h_01H8YCQKJ1YPHZ5KBP9G8GCJRD
 */
@Serializable
public data class UserRecordingSession(
  val duration: String = "",

  @SerialName("session_name")
  val sessionName: String = "",

  @SerialName("start_time")
  val startTime: String = "",

  @SerialName("start_timestamp")
  val startTimestamp: Long = -1,

  @SerialName("session_description")
  val sessionDescription: String = "",

  @SerialName("session_creator")
  val sessionCreator: String = "",

  @SerialName("repeat_summary")
  val repeatSummary: String = "",

  @SerialName("device_profile")
  val deviceProfile: String = "",

  @SerialName("session_id")
  val sessionId: Long = -1,

  @SerialName("class_name")
  val className: String = "",

  @SerialName("class_code")
  val classCode: String = "",

  @SerialName("video_id")
  val videoId: Long = -1,
)

@Serializable
public data class DeviceRecordingSession(
  @SerialName("session_name")
  val sessionName: String = "",

  val isLiveStream: Boolean = false,

  @SerialName("start_timestamp")
  val startTimestamp: Long = -1,

  @SerialName("session_creator")
  val sessionCreator: String = "",

  @SerialName("owner_id")
  val ownerId: Long = -1,

  @SerialName("device_profile")
  val deviceProfile: String = "",

  @SerialName("session_id")
  val sessionId: Long = -1,

  val duration: String,

  @SerialName("start_time")
  val startTime: String = "",

  @SerialName("session_description")
  val sessionDescription: String = "",

  @SerialName("repeat_summary")
  val repeatSummary: String = "",

  @SerialName("class_name")
  val className: String = "",

  @SerialName("class_code")
  val classCode: String = "",

  @SerialName("platform_unique_id")
  val platformUniqueId: String = "",

  @SerialName("video_id")
  val videoId: Long = -1,
)

@Serializable
public data class Device(
  @SerialName("station_name")
  val stationName: String = "",

  @SerialName("access_level")
  val accessLevel: String = "",

  @SerialName("device_id")
  val deviceId: Long = -1,

  @SerialName("access_id")
  val accessId: String = "",

  @SerialName("station_type")
  val stationType: String = "",
)

internal data class DeviceRegistration(
  val name: String = "",

  @SerialName("device_type")
  val deviceType: String = "",

  @SerialName("access_level")
  val accessLevel: String = "",

  @SerialName("device_id")
  val deviceId: Long = -1,

  @SerialName("register_user_id")
  val registerUserId: String = "",
)

@Serializable
public data class Group(
  @SerialName("embed_code")
  val embedCode: String = "",

  @SerialName("publish_type")
  val publishType: String = "",

  @SerialName("is_active")
  val isActive: Boolean = false,

  @SerialName("group_code")
  val groupCode: String = "",

  @SerialName("group_id")
  val groupId: Long = -1,

  @SerialName("group_name")
  val groupName: String = "",

  @SerialName("owner_id")
  val ownerId: Long = -1,

  @SerialName("sis_id")
  val sisId: String = "",

  @SerialName("group_security")
  val groupSecurity: String = "",

  @SerialName("group_term")
  val groupTerm: String = "",

  @SerialName("direct_link")
  val directLink: String = "",
)

@Serializable
public data class Course(
  @SerialName("embed_code")
  val embedCode: String = "",

  @SerialName("course_id")
  val courseId: Long = -1,

  @SerialName("publish_type")
  val publishType: String = "",

  @SerialName("is_active")
  val isActive: Boolean = false,

  @SerialName("course_code")
  val courseCode: String = "",

  @SerialName("course_name")
  val courseName: String = "",

  @SerialName("owner_id")
  val ownerId: Long = -1,

  @SerialName("sis_id")
  val sisId: String = "",

  @SerialName("course_security")
  val courseSecurity: String = "",

  @SerialName("course_term")
  val courseTerm: String = "",

  @SerialName("direct_link")
  val directLink: String = "",
)

@Serializable
public data class AuditLogEntry(
  @Serializable(with = LocalDateTimeSerializer::class)
  val creationTimestamp: LocalDateTime? = null,

  val eventName: String = "",

  val details: String = "",
)

@Serializable
public data class StorageAnalyticsEntry(
  public val mediaType: String = "",

  public val date: LocalDate,

  public val count: Long,

  public val size: Long,
)
