/*
 *     Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 *     Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package edu.ucsc.its.temerity

import co.touchlab.kermit.Logger
import io.kotest.core.spec.style.FunSpec
import kotlinx.serialization.Serializable
import kotlinx.serialization.json.Json
import org.dotenv.vault.dotenvVault
import org.jetbrains.kotlinx.dataframe.DataFrame
import org.jetbrains.kotlinx.dataframe.api.any
import org.jetbrains.kotlinx.dataframe.api.filter
import org.jetbrains.kotlinx.dataframe.api.first
import org.jetbrains.kotlinx.dataframe.api.isNotEmpty
import org.jetbrains.kotlinx.dataframe.api.rows
import org.jetbrains.kotlinx.dataframe.api.toDataFrame
import org.jetbrains.kotlinx.dataframe.io.read
import org.jetbrains.kotlinx.dataframe.io.toJson
import org.jetbrains.kotlinx.dataframe.io.writeCSV
import java.util.UUID

class CanvasYuJaDiffReport : FunSpec({

  @Serializable
  data class DataFrameUser(
    val loginId: String = "",
    val emailAddress: String = "",
    val userType: String = "",
    val userId: Long = -1,
    val timezone: String = "",
    val customId: String = "",
    var lastName: String = "",
    val phoneNumber: String = "",
    val firstName: String = "",
    var canvasEmail: String = "",
  )

  val dotEnv = dotenvVault()

  test("compare canvas and yuja users") {
    val yujaUsersList = DataFrame.read(dotEnv["YUJA_USERS_FILE"])
    val yujaInstructorsList = yujaUsersList.rows().filter { it["userType"] == "Instructor" }
    val canvasUsersList = DataFrame.read(dotEnv["CANVAS_USERS_FILE"])

    canvasUsersList.rows().forEach { row ->
      if (row["created_by_sis"] == "TRUE") {
        val canvasLoginId = row["login_id"] as? String
        val canvasLastName = row["last_name"] as? String
        val canvasFirstName = row["first_name"] as? String

        // Check if 'login_id' exists in 'yujaUsersList'
        val loginIdExists = yujaInstructorsList.any { yujaUserRow -> yujaUserRow["emailAddress"] == canvasLoginId }

        // If not, check if 'last_name' and 'first_name' exist in 'yujaUsersList' possibly under different 'login_id'
        if (!loginIdExists) {
          val possibleExtraYuJaUsers = yujaInstructorsList.filter {
            it["lastName"] == canvasLastName
          }.filter {
            it["firstName"] == canvasFirstName
          }
          if (possibleExtraYuJaUsers.isNotEmpty()) {
            Logger.i { "Found YuJa users with different login id than Canvas user $canvasLoginId: $possibleExtraYuJaUsers" }
          }
          Logger.i { "Canvas user $canvasLoginId not found in YuJa" }
        } else {
          // Check if names in YuJa don't match Canvas
          val yuJaUserNamesAreDifferent = yujaInstructorsList.any {
            it["emailAddress"] == canvasLoginId &&
              it["lastName"] != canvasLastName &&
              it["firstName"] != canvasFirstName
          }
          if (yuJaUserNamesAreDifferent) {
            Logger.i {
              "Canvas user $canvasLoginId has different first name or last name in YuJa: ${yujaInstructorsList.filter {
                it["emailAddress"] == canvasLoginId &&
                  it["lastName"] != canvasLastName &&
                  it["firstName"] != canvasFirstName
              }}"
            }
          }

          // Check if email in YuJa doesn't match Canvas
          val yujaUserEmailIsDifferent = yujaInstructorsList.any {
            it["emailAddress"] != canvasLoginId &&
              it["lastName"] == canvasLastName &&
              it["firstName"] == canvasFirstName
          }
          if (yujaUserEmailIsDifferent) {
            Logger.i {
              "Canvas user $canvasLoginId has different YuJa login email: ${yujaInstructorsList.first {
                it["emailAddress"] != canvasLoginId &&
                  it["lastName"] == canvasLastName &&
                  it["firstName"] == canvasFirstName
              }["emailAddress"]}"
            }
          }
        }
      }
    }
  }

  test("Log users whose email exists in YuJa but not Canvas") {
    val json = Json {
      coerceInputValues = true
    }
    val yujaUsersList = DataFrame.read(dotEnv["YUJA_USERS_FILE"])
    val canvasUsersList = DataFrame.read(dotEnv["CANVAS_USERS_FILE"])
    val unResolvedYuJaUsers = ArrayList<DataFrameUser>()

    yujaUsersList["emailAddress"].values().forEachIndexed { index, yujaEmail ->
      if (yujaEmail != null && ((yujaEmail as? String) != "")) {
        val yuJaUserRecord = yujaUsersList.rows().first { it["emailAddress"] == yujaEmail }
        val emailExistsInCanvas = canvasUsersList.any { it["login_id"] == yujaEmail }

        if (!emailExistsInCanvas) {
          Logger.i { "YuJa user $yujaEmail not found in Canvas" }
          val unResolvedUser = json.decodeFromString<DataFrameUser>(yuJaUserRecord.toJson())
          val guessedCanvasUsers = canvasUsersList.filter {
            it["last_name"] == unResolvedUser.lastName &&
              it["first_name"] == unResolvedUser.firstName
          }
          if (guessedCanvasUsers.isNotEmpty()) {
            unResolvedUser.canvasEmail = guessedCanvasUsers.first()["login_id"] as? String ?: ""
          }
          unResolvedYuJaUsers.add(unResolvedUser)
        }
      }
    }

    val df = unResolvedYuJaUsers.toDataFrame()
    df.writeCSV("unresolved_yuja_users_${UUID.randomUUID()}_${getReportDate()}.csv")
  }
})
