/*
 *     Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 *     Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package edu.ucsc.its.temerity
import io.kotest.core.spec.style.FunSpec
import io.kotest.matchers.file.shouldNotBeEmpty
import kotlinx.coroutines.runBlocking
import kotlinx.datetime.Clock
import kotlinx.datetime.DateTimeUnit
import kotlinx.datetime.TimeZone
import kotlinx.datetime.minus
import kotlinx.datetime.todayIn
import org.dotenv.vault.dotenvVault
import org.jetbrains.kotlinx.dataframe.api.toDataFrame
import org.jetbrains.kotlinx.dataframe.io.writeCSV
import java.io.File
import java.util.UUID
import kotlin.time.Duration.Companion.minutes

class ProdReportTests : FunSpec() {
  init {
    coroutineDebugProbes = true

    val dotenv = dotenvVault()

    lateinit var classUnderTest: PlatformClient
    val today = Clock.System.todayIn(TimeZone.currentSystemDefault())

    beforeTest {
      classUnderTest = platformClient {
        debugEnabled(true)
        serviceEndpoint(dotenv["YUJAPROD_API_URL"])
        serviceToken(dotenv["YUJAPROD_TOKEN"])
      }
    }

    timeout = 5.minutes.inWholeMilliseconds

    test("PlatformClient can fetch a list of devices") {
      runBlocking {
        val returnedDevices = classUnderTest.getDevices()
        println("Returned devices:")
        returnedDevices.forEach { println(it) }
        val df = returnedDevices.toDataFrame()
        df.writeCSV("YuJaDevices-${getReportDate()}.csv")
        assert(returnedDevices.isNotEmpty())
      }
    }

    // Generate Report listing all groups
    test("PlatformClient can generate Groups report") {
      runBlocking {
        val returnedGroups = classUnderTest.getGroups()
        println("Total returned groups count: ${returnedGroups.size}")
        val df = returnedGroups.toDataFrame()
        df.writeCSV("YuJaGroups.csv")
        assert(returnedGroups.isNotEmpty())
      }
    }

    test("PlatformClient can generate Group terms list report") {
      runBlocking {
        val returnedGroups = classUnderTest.getGroups()
        println("Total returned groups count: ${returnedGroups.size}")
        val termList = returnedGroups.map { it.groupTerm }.distinct()
        val df = termList.toDataFrame()
        df.writeCSV("YuJaTermsList.csv")
        assert(returnedGroups.isNotEmpty())
      }
    }

    test("PlatformClient can generate User list report") {
      runBlocking {
        val returnedUsers = classUnderTest.getUsers()
        println("Total returned user count: ${returnedUsers.size}")
        val df = returnedUsers.toDataFrame()
        df.writeCSV("YuJaUsersList-${getReportDate()}-${UUID.randomUUID()}.csv")
        assert(returnedUsers.isNotEmpty())
      }
    }

    // Generate Report listing all groups matching specific term code
    test("PlatformClient can fetch a list of groups from Spring Quarter 2024") {
      runBlocking {
        val returnedGroups = classUnderTest.getGroups().filter { it.groupTerm == "2024 Spring Quarter" }
        println("Total returned groups count: ${returnedGroups.size}")
        returnedGroups.forEach { println(it) }
        val df = returnedGroups.toDataFrame()
        df.writeCSV("2024SpringQuarterGroups.csv")
        assert(returnedGroups.isNotEmpty())
      }
    }

    // Generate Report listing all groups matching specific term code
    test("PlatformClient can fetch a list of courses from Summer Quarter 2024") {
      runBlocking {
        val returnedCourses = classUnderTest.getCourses().filter { it.courseTerm == "2024 Summer Quarter" }
        println("Total returned courses count: ${returnedCourses.size}")
        returnedCourses.forEach { println(it) }
        val df = returnedCourses.toDataFrame()
        df.writeCSV("2024SummerQuarterCourses.csv")
        assert(returnedCourses.isNotEmpty())
      }
    }

    test("PlatformClient can fetch a list of groups from sis term code 2242") {
      runBlocking {
        val returnedGroups = classUnderTest.getGroups().filter { it.sisId.startsWith("2242") }
        println("Total returned groups count: ${returnedGroups.size}")
        returnedGroups.forEach { println(it) }
        val df = returnedGroups.toDataFrame()
        df.writeCSV("2024SpringQuarterGroups-${getReportDate()}.csv")
        assert(returnedGroups.isNotEmpty())
      }
    }

    test("PlatformClient can fetch a list of groups from sis term code 2240 and verify term code matches") {
      runBlocking {
        val returnedGroups = classUnderTest.getGroups().filter {
          it.sisId.startsWith("2240")
        }.filter {
          it.groupTerm != "2024 Winter Quarter"
        }
        println("Total returned groups count: ${returnedGroups.size}")
        returnedGroups.forEach { println(it) }
        val df = returnedGroups.toDataFrame()
        df.writeCSV("2024WinterQuarterGroups-WrongTerm-${getReportDate()}.csv")
        assert(returnedGroups.isNotEmpty())
      }
    }

    // "Mislinked" YuJa course group report
    // Generate Report listing all groups matching specific sis id that have the wrong term code, or are duplicates
    test("PlatformClient can fetch a list of groups with specific SIS ID by term and verifying term code matches") {
      runBlocking {
        val invalidGroupTerm = classUnderTest.getGroups().filter {
          it.sisId.startsWith("2242")
        }.filter {
          it.groupTerm != "2024 Spring Quarter"
        }
        val nonMatchingSisId = classUnderTest.getGroups().filter {
          it.groupTerm == "2024 Spring Quarter"
        }.filter {
          !it.sisId.startsWith("2242")
        }
        val duplicateSisId = classUnderTest.getGroups().filter {
          it.sisId.startsWith("2242")
        }.groupBy { it.sisId }.filter { it.value.size > 1 }.values.flatten()
        val returnedGroups = invalidGroupTerm + nonMatchingSisId + duplicateSisId
        println("Total returned groups count: ${returnedGroups.size}")
        returnedGroups.forEach { println(it) }
        val df = returnedGroups.toDataFrame()
        df.writeCSV("wrong-term-duplicate-course-report-${UUID.randomUUID()}-${getReportDate()}.csv")
        assert(returnedGroups.isNotEmpty())
      }
    }

    test("YuJaClient can fetch the whole list of groups") {
      runBlocking {
        val returnedGroups = classUnderTest.getGroups()
        println("Total returned groups count: ${returnedGroups.size}")
        returnedGroups.forEach { println(it) }
        assert(returnedGroups.isNotEmpty())
      }
    }

    test("Get all audit log entries of type \"New Login\" from the past day") {
      runBlocking {
        val testEventType = EventType.NEW_LOG_IN
        val currentDate = Clock.System.todayIn(TimeZone.currentSystemDefault())
        val pastDate = currentDate.minus(1, DateTimeUnit.DAY)

        val returnedAuditLogEntries = classUnderTest.getAuditLogEntries(
          startTime = pastDate,
          endTime = currentDate,
          eventType = testEventType,
          sortOrder = AuditLogSortOrder.NEW_FIRST,
        )
        returnedAuditLogEntries.forEach { println(it) }
        val df = returnedAuditLogEntries.toDataFrame()

        df.writeCSV("${testEventType.name}-Event-Report-${UUID.randomUUID()}-$pastDate-$currentDate-generated-${getReportDate()}.csv")
        assert(returnedAuditLogEntries.isNotEmpty())
      }
    }

    test("Get and compile device audit log entries of multiple types from the past 2 years") {
      runBlocking {
        val testEventTypeList = listOf(
          EventType.AUTOMATED_SESSION_FAILED_TO_START,
          EventType.ERROR,
          EventType.RECORDING_ERROR,
          EventType.DEVICE_FAILED_TO_START,
          EventType.DEVICE_ERROR,
          EventType.HUB_CAPTURE_PROBLEM,
          EventType.HDCP_SIGNAL_DETECTED,
          EventType.CAPTURE_ERROR,
          EventType.HARDWARE_CAPTURE_HUB,
        )
        val pastDate = today.minus(2, DateTimeUnit.YEAR)
        val logEntries = classUnderTest.getAuditLogEntries(
          startTime = pastDate,
          endTime = today,
          eventTypeList = testEventTypeList,
        )
        val df = logEntries.toDataFrame()
        val saveFilename = "DeviceAndSessionError-Event-Report-$pastDate-$today-generated-${getReportDate()}-${UUID.randomUUID()}.csv"
        df.writeCSV(saveFilename)
        File(saveFilename).shouldNotBeEmpty()
      }
    }

    test("Dump all audit log reports of type \"Automated Session Monitor\" from the past week") {
      runBlocking {
        val testEventType = EventType.AUTOMATED_SESSION_MONITOR
        val windowStart = today.minus(1, DateTimeUnit.WEEK)
        val logEntries = classUnderTest.getAuditLogEntries(
          startTime = windowStart,
          endTime = today,
          eventType = testEventType,
        )
        val df = logEntries.toDataFrame()
        val saveFilename = "${testEventType.name}-Event-Report-${UUID.randomUUID()}-$windowStart-$today-generated-${getReportDate()}.csv"
        df.writeCSV(saveFilename)
        File(saveFilename).shouldNotBeEmpty()
      }
    }
  }
}
